
#####################
##### Libraries #####
#####################

library(GENEAread)
library(GGIR)
library(tidyverse)
library(vroom)
library(summarytools)
library(gt)
library(gtsummary)
library(webshot2)
library(colorspace)
library(geomtextpath)


########################################################
##### Calibration of raw acceleration measurements #####
########################################################

# title: "GENEActiv calibration 09-12-21"
# authors: "Charles Sweetland, Joss Langford"
# copyright: "Activinsights Ltd, 2021"

# Clear the decks
rm(list = ls())
gc()

Sys.Date()

# store initial working directory
Basedir = getwd()

# Set Data folder
Data = paste0(Basedir,"/Data/")

# Set Uncal folder
Uncal = paste0(Basedir,"/Uncal/")

# Finding the bin files inside the Input folder
BinPattern <- "*\\.[bB][iI][nN]$"
files = list.files(path = Uncal, pattern = BinPattern, full.names = TRUE, recursive = TRUE)
NoFiles = length(files)
if (NoFiles == 0){stop("There are no files in the data directory")}

# Initialise parameters
Scale_X = Scale_Y = Scale_Z = Offset_X = Offset_Y = Offset_Z = TempOffset_X = TempOffset_Y = TempOffset_Z = QCMessage = c()

#Binfile[j] = files[j]
filenames = gsub(".bin", "", basename(files))

for (j in 1:NoFiles){ 
  Binfiles = files[j]
  
  # Running the calibration function 
  C = GENEActiv.calibrate(Binfiles, 
                          use.temp = TRUE,
                          spherecrit = 0.3, 
                          minloadcrit = 72,
                          printsummary = FALSE, 
                          chunksize=c(0.5), 
                          windowsizes=c(60,900,3600))
  
  if (C$QCmessage != "recalibration not done because not enough points on all sides of the sphere"){
    #### Then adding this to the overall csv file writing this out to a csv file #### 
    digits = 6
    Scale_X[j] = round(C$scale[1], digits = digits)
    Scale_Y[j] = round(C$scale[2], digits = digits)
    Scale_Z[j] = round(C$scale[3], digits = digits)
    # Offset
    Offset_X[j] = round(C$offset[1], digits = digits)
    Offset_Y[j] = round(C$offset[2], digits = digits)
    Offset_Z[j] = round(C$offset[3], digits = digits)
    # Temp offset
    TempOffset_X[j] = round(C$tempoffset[1], digits = digits)
    TempOffset_Y[j] = round(C$tempoffset[2], digits = digits)
    TempOffset_Z[j] = round(C$tempoffset[3], digits = digits)
    
    # Message 
    QCMessage[j] = as.character(C$QCmessage[1])
    
    #### Creating a single csv for each file ####
    digits = 6
    df = data.frame(digits = 6,
                    file = Binfiles,
                    Scale_X = Scale_X[j],
                    Scale_Y = Scale_Y[j],
                    Scale_Z = Scale_Z[j],
                    # Offset
                    Offset_X = Offset_X[j],
                    Offset_Y = Offset_Y[j],
                    Offset_Z = Offset_Z[j],
                    # Temp offset
                    TempOffset_X = TempOffset_X[j],
                    TempOffset_Y = TempOffset_Y[j],
                    TempOffset_Z = TempOffset_Z[j],
                    
                    QCMessage = QCMessage[j])
  } else {
    
    digits = 6      
    Scale_X[j] = round(C$scale[1], digits = digits)
    Scale_Y[j] = round(C$scale[2], digits = digits)
    Scale_Z[j] = round(C$scale[3], digits = digits)
    # Offset
    Offset_X[j] = round(C$offset[1], digits = digits)
    Offset_Y[j] = round(C$offset[2], digits = digits)
    Offset_Z[j] = round(C$offset[3], digits = digits)
    # Temp offset
    TempOffset_X[j] = NA
    TempOffset_Y[j] = NA
    TempOffset_Z[j] = NA
    
    # Message 
    QCMessage[j] = as.character(C$QCmessage[1])
    
    #### Creating a single csv for each file ####
    digits = 6
    df = data.frame(digits = 6,
                    file = Binfiles,
                    Scale_X = Scale_X[j],
                    Scale_Y = Scale_Y[j],
                    Scale_Z = Scale_Z[j],
                    # Offset
                    Offset_X = Offset_X[j],
                    Offset_Y = Offset_Y[j],
                    Offset_Z = Offset_Z[j],
                    # Temp offset
                    TempOffset_X = TempOffset_X[j],
                    TempOffset_Y = TempOffset_Y[j],
                    TempOffset_Z = TempOffset_Z[j],
                    
                    QCMessage = QCMessage[j])
  }
  
  
  # Write this CSV out if needed
  #write.csv(df, paste0(Data,filenames[j],"_Calibration_Values.csv"))
  
  # Read in the current values - 
  Lines=readLines(Binfiles,-1) # Reads the bin file to the point where the calibration data is.
  XOffset = Lines[49];  XGain = Lines[48]
  YOffset = Lines[51];  YGain = Lines[50]
  ZOffset = Lines[53];  ZGain = Lines[52]
  
  # Extracting just the numerical value. - Offset
  XOffsetN = as.numeric(unlist(strsplit(XOffset,"x offset:"))[2])
  YOffsetN = as.numeric(unlist(strsplit(YOffset,"y offset:"))[2])
  ZOffsetN = as.numeric(unlist(strsplit(ZOffset,"z offset:"))[2])
  
  # Extracting just the numerical value for the Gain
  XGainN = as.numeric(unlist(strsplit(XGain,"x gain:"))[2])
  YGainN = as.numeric(unlist(strsplit(YGain,"y gain:"))[2])
  ZGainN = as.numeric(unlist(strsplit(ZGain,"z gain:"))[2])
  
  # Calculating the New values using the calibration values- Rounding to whole numbers
  if (C$offset[1] != 0){
    #        XOffsetNew = round((XOffsetN * C$offset[1]) * 25.6)
    XOffsetNew = round(XOffsetN - C$offset[1] * 256 * 100)
    Lines[49] = paste("x offset:", sep = "", XOffsetNew)}
  
  if (C$offset[2] != 0){
    #        YOffsetNew = round((YOffsetN * C$offset[2]) * 25.6)
    YOffsetNew = round(YOffsetN -  C$offset[2] * 256 * 100)
    Lines[51] = paste("y offset:", sep = "", YOffsetNew)
  }
  
  if (C$offset[2] != 0){
    #        ZOffsetNew = round((ZOffsetN * C$offset[3]) * 25.6)
    ZOffsetNew = round(ZOffsetN - C$offset[3] * 256 * 100)
    Lines[53] = paste("z offset:", sep = "", ZOffsetNew)
  }
  
  if (C$scale[1] != 0){
    XGainNew = round(XGainN / C$scale[1])
    Lines[48] = paste("x gain:", sep = "", XGainNew)
  }
  
  if (C$scale[2] != 0){
    YGainNew = round(YGainN / C$scale[2])
    Lines[50] = paste("y gain:", sep = "", YGainNew)
  }
  if (C$scale[3] != 0){
    ZGainNew = round(ZGainN / C$scale[3])
    Lines[52] = paste("z gain:", sep = "", ZGainNew)
  }
  
  # Create the calibrated file name
  con <- file(paste0(Data, filenames[j], "_Calibrated.bin"))
  write(Lines, con)
  close (con)
} 

Stats = data.frame(filenames,
                   Scale_X, 
                   Scale_Y, 
                   Scale_Z, 
                   Offset_X, 
                   Offset_Y, 
                   Offset_Z, 
                   TempOffset_X, 
                   TempOffset_Y, 
                   TempOffset_Z, 
                   QCMessage)

colnames(Stats) = c("File",
                    "Scale_X",
                    "Scale_Y", 
                    "Scale_Z", 
                    "Offset_X", 
                    "Offset_Y", 
                    "Offset_Z", 
                    "TempOffset_X", 
                    "TempOffset_Y", 
                    "TempOffset_Z", 
                    "Message")

# Writing out the CSV 
write.csv(Stats, 
          paste0(Data, "Calibration_Values.csv"), 
          row.names = F)



#########################################
##### GENEActiv raw data processing #####
#########################################

# A copy of the latest analysis tool for batch-processing of raw accelerometer data
# with the open-source GENEAcount algorithm can be obtained from:
# https://activinsights.com/support/geneactiv-support/



########################################
##### GGIR processing (valid days) #####
########################################

# Process binary data with GGIR parts 1 and 2 to identify valid hours in each day
GGIR(datadir = 'LINK TO CALIBRATED GENEACTIV RAW DATA', # Path to folder with calibrated raw data
     outputdir = 'LINK TO OUTPUT FOLDER',
     mode = 1:2, # Run GGIR parts 1 and 2
     do.report = 2, # Generate csv-report for GGIR part 2
     qwindow = c(0, 24)) # Analyse midnight to midnight (full day segments only)



##########################
###### Read in data ######
##########################


### GENEActiv

# Prepare file path to folder with all processed GENEActiv .csv output from REACT study
accel_files <- fs::dir_ls(path = 'LINK TO FOLDER CONTAINING "ALL DATA" CSVs FROM MARKDOWN OUTPUT')

# Read GENEActiv .csv files and add an ID column with file path names
accel_orig <- vroom(accel_files,
                    delim = ',',
                    id = 'path',
                    # Pre-select relevant columns for memory management
                    col_select = c(source, segment_duration, step_gene_acount, date),
                    .name_repair = ~ janitor::make_clean_names(., case = 'snake'))

# Save a copy as .rds to avoid re-reading all CSVs in the future
saveRDS(accel, 'accel.rds')
# Reading in the .rds will be much faster than re-loading the .csv files later
accel_orig <- readRDS(file = 'LINK TO OUTPUT FOLDER TO SAVE accel_orig.rds')



### GGIR
# Read in GGIR valid day results
ggir <- vroom('LINK TO "PART2 DAYSUMMARY" FROM GGIR',
              delim = ',',
              col_types = cols(calendar_date = col_date(format = '%Y-%m-%dT%Z')),
              .name_repair = ~ janitor::make_clean_names(., case = 'snake'))



#### REACT study lab data
# Read first file
lab <- vroom('LINK TO REACT LABORATORY RESULTS CSV',
             delim = ',',
             .name_repair = ~ janitor::make_clean_names(., case = 'snake'))
# Read second file
lab_part2 <- vroom('LINK TO REACT LABORATORY RESULTS CSV PART 2',
                   delim = ',',
                   .name_repair = ~ janitor::make_clean_names(., case = 'snake')) %>%
  select(-c(imd_rank, imd_decile))
# Combine files
lab <- left_join(lab, lab_part2,
                 by = c('site_person_id', 'visit')) %>%
  # Remove variables which aren't required for memory management
  select(-c(tot_bouts, tot_vol, mean_dur, med_dur, mean_vol, med_vol, mean_int, med_int,
            peak_int, bet_day_stab_is, with_day_var_iv, vol_day1, vol_day2, vol_day3, 
            vol_day4, vol_day5, vol_day6, vol_day7, tot_bouts_z, tot_vol_z, med_dur_z, 
            med_vol_z, mean_int_z, med_int_z, peak_int_z, bet_day_stab_is_z, 
            with_day_var_iv_z, mean_vol_z, age_z, atten_dpct_wk0to12_excl70, 
            atten_dpct_wk13to52_excl70, atten_dpct_all_wks_excl_zeros,
            m4walk_score_4v012, m4walk_score_4v3v012, m4walk_score_34v012))



###############################
###### Clean data frames ######
###############################

### Clean accel data frame
accel_orig <- accel_orig %>%
  # Remove unnecessary variables
  select(c('path', 'source', 'segment_duration', 'step_gene_acount', 'date')) %>%
  # Simplify variables names
  rename(num_steps = step_gene_acount,
         duration_s = segment_duration) %>%
  relocate(date,
           .before = duration_s) %>%
  # Turn file path column into ID for the four time points of measurement
  mutate(path = str_extract(path, '_[A-D]_')) %>%
  rename(visit = path) %>%
  # Make time point indicators more descriptive
  mutate(visit = case_when(visit == '_A_' ~ 'Baseline',
                           visit == '_B_' ~ '6 Months',
                           visit == '_C_' ~ '12 Months',
                           visit == '_D_' ~ '24 Months')) %>%
  # Define time point variable factors
  mutate(visit = factor(visit, 
                        levels = c('Baseline', '6 Months', '12 Months', '24 Months'))) %>%
  # Copy "source" column and extract site name from the file name
  mutate(site = source,
         site = str_extract(site, 'Birmingham|Bath|Exeter')) %>%
  relocate(site,
           .before = visit) %>%
  # Turn original "source" column into participant ID based on the file name
  mutate(source = str_extract(source, '[ABE][T]\\d{4}')) %>%
  rename(participant_id = source) %>%
  # Define factors
  mutate(participant_id = factor(participant_id),
         site = factor(site))

### Clean GGIR output
ggir <- ggir %>%
  # Remove unnecessary variables
  select('filename', 'calendar_date', 'n_valid_hours', 'n_hours', 'weekday', 'measurementday') %>%
  # Turn "filename" column into participant ID
  mutate(filename = str_extract(filename, '[ABE][T]\\d{4}')) %>%
  # Clean up variable names
  rename(participant_id = filename,
         measurement_day = measurementday) %>%
  # Reorder variables logically
  relocate(weekday,
           .after = calendar_date) %>%
  relocate(measurement_day,
           .after = weekday) %>%
  relocate(n_hours,
           .before = n_valid_hours) %>%
  # Make calendar_date a date-only column (remove time component)
  mutate(date = as.Date(calendar_date), 
         .after = calendar_date) %>%
  select(-calendar_date) %>%
  # Add column with valid/invalid day indicator with max. 6hrs daily non-wear
  mutate(valid_day = ifelse(n_valid_hours < 18, 'invalid', 'valid')) %>%
  # Define factors
  # Measurement day is purposefully not treated as factor at this stage to enable 
  # filtering further down the line
  mutate(weekday = factor(weekday, 
                          levels = c('Monday', 'Tuesday', 'Wednesday', 'Thursday',
                                     'Friday', 'Saturday', 'Sunday')),
         participant_id = factor(participant_id),
         valid_day = factor(valid_day))

### Clean lab-based physical activity and health data
lab <- lab %>%
  # Correct variable names
  rename(age = age_base,
         participant_id = allocation_id,
         allocation = allocation_int1con0,
         final_sppb_score = final_spp_bscore,
         final_sppb_score_count = final_spp_bscore_count,
         final_sppb_score_cat = final_spp_bscore_cat,
         study_loss_to_follow_up = study_losstofollowup,
         pa_visit_count = p_avisit_count,
         pa_days_count = p_adays_count,
         highest_education = educ_5levels,
         comorbid = poor_health_yes) %>%
  # Reorder variables logically
  relocate(imd_quintile,
           .after = imd_rank) %>%
  
  # Remove observations for which no participant ID is available because
  # these participants were not randomised after baseline measurement
  drop_na(participant_id) %>%
  # Remove participants who don't have data for all model variables
  drop_na(gender, age, allocation, final_sppb_score, sf36gen_hlth, 
          comorbid, imd_quintile, highest_education) %>%
  # Correct the 4m walking scores for those who did a 3m walk instead (the coding from the
  # original REACT data is wrong in those instances)
  mutate(m4walk_score = case_when(participant_id == 'BT1537' & visit == '6 Months' ~ 2,
                                  participant_id == 'AT2072' & visit == '24 Months' ~ 1,
                                  participant_id == 'ET1073' & visit == '24 Months' ~ 1,
                                  TRUE ~ m4walk_score)) %>%
  # Update counts to include the changes in the preceding four lines
  group_by(site_person_id) %>%
  # Correct the pa_visit_count column which incorrectly gave everyone a 4 in the original data
  # Count only visits where six or more days of activity data were available
  mutate(pa_visit_count = length(na.omit(pa_days_count >= 6)),
         # Correct the m4walk_score_count
         m4walk_score_count = length(na.omit(m4walk_score))) %>%
  ungroup() %>%
  
  # Re-code variables and set factor levels
  # Make the visit number match the visit variable in the other data frames for merging
  mutate(visit = case_when(visit == '1' ~ 'Baseline',
                           visit == '2' ~ '6 Months',
                           visit == '3' ~ '12 Months',
                           visit == '4' ~ '24 Months'),
         visit = factor(visit, 
                        levels = c('Baseline', '6 Months', '12 Months', '24 Months')),
         # Re-code the gender variable to be more intuitive
         gender = case_when(gender == '1' ~ 'Male',
                            gender == '2' ~ 'Female'),
         gender = factor(gender,
                         levels = c('Male', 'Female')),
         # Re-code the allocation variable to be more intuitive
         allocation = case_when(allocation == '0' ~ 'Control',
                                allocation == '1' ~ 'Intervention'),
         allocation = factor(allocation,
                             levels = c('Control', 'Intervention')),
         # Condense sf36_health, highest_education, and highest_education variables because there 
         # are few participants in the extremes (1 or 5) and set factor levels. Plus, recode 
         # imd_quintile with more meaningful labels
         sf36gen_hlth = case_when(sf36gen_hlth == 1 | sf36gen_hlth == 2 ~ 'Very good/Excellent',
                                  sf36gen_hlth == 3 ~ 'Good',
                                  sf36gen_hlth == 4 | sf36gen_hlth == 5 ~ 'Poor/Fair'),
         sf36gen_hlth = factor(sf36gen_hlth,
                               levels = c('Very good/Excellent', 'Good', 'Poor/Fair')),
         # Comorbidities
         comorbid = case_when(comorbid == 0 ~ 'None',
                              comorbid >= 1 ~ 'One or more'),
         comorbid = factor(comorbid,
                           levels = c('None', 'One or more')),
         # Highest education
         highest_education = case_when(highest_education == 1 | highest_education == 2 ~ 'Some/all Secondary',
                                       highest_education == 3 ~ 'Some College',
                                       highest_education == 4 | highest_education == 5 ~ 'All College/Degree'),
         highest_education = factor(highest_education,
                                    levels = c('Some/all Secondary', 'Some College', 
                                               'All College/Degree')),
         # IMD quintile
         imd_quintile = case_when(imd_quintile == 1 ~ 'Most deprived (1/5)',
                                  imd_quintile == 2 ~ 'Quite deprived (2/5)',
                                  imd_quintile == 3 ~ 'Mid deprived (3/5)',
                                  imd_quintile == 4 ~ 'Little deprived (4/5)',
                                  imd_quintile == 5 ~ 'Least deprived (5/5)'),
         imd_quintile = factor(imd_quintile,
                               levels = c('Most deprived (1/5)', 'Quite deprived (2/5)',
                                          'Mid deprived (3/5)', 'Little deprived (4/5)',
                                          'Least deprived (5/5)')),
         # Convert variables where no specific levels are needed to factor 
         across(c(site_person_id, participant_id, m4walk_score, chair_rise_score, age_gt80,
                  balance_score, final_sppb_score, final_sppb_score_cat, imd_decile), 
                as.factor))



###########################################################
##### Merge data frames and apply exclusion criteria ######
###########################################################

accel <- accel_orig %>%
  ### Join GGIR valid day information to the accelerometer observations
  inner_join(ggir,
             by = c('participant_id', 'date')) %>%
  # Move informative variables to front of data frame
  relocate(c(measurement_day, weekday, n_hours, n_valid_hours), 
           .after = date) %>%
  
  ### Join in lab data
  inner_join(lab,
             by = c('participant_id', 'visit')) %>%
  # Move informative variables to front
  relocate(c(allocation, age, gender),
           .after = participant_id) %>%
  
  ### Calculate event cadences in steps per minute
  mutate(cadence = num_steps / duration_s * 60,
         .after = duration_s) %>%
  
  ### Apply exclusion criteria
  # Remove all events with less than 10 consecutive steps
  filter(num_steps >= 10) %>%
  # Filter out invalid cadences 
  filter(cadence >= 20, # < 20 steps/min not valid estimates
         cadence <= 175) %>% # > 175 steps/min represent workouts, not walking
  # Remove invalid days
  filter(valid_day == 'valid') %>%
  # Remove valid day indicator now that it's no longer useful
  select(-valid_day) %>%
  # Remove all days beyond the seventh measurement day
  filter(measurement_day <= 7) %>%
  # Count the remaining valid days for each participant at each visit
  group_by(participant_id, visit) %>%
  mutate(days_per_visit = n_distinct(date),
         .after = gender) %>%
  ungroup() %>%
  # Remove pa_days_count variable which is superseded by days_per_visit variable
  select(-pa_days_count) %>%
  # Remove visits where a participant has less than 6 valid days of accelerometer data left
  filter(days_per_visit >= 6) %>%
  # Calculate the number of visits with accelerometer data for each participant
  group_by(participant_id) %>%
  mutate(num_accel_visits = n_distinct(na.omit(visit)),
         .after = gender) %>%
  ungroup() %>%
  # Remove pa_visit_count variable which is superseded by num_accel_visits
  select(-pa_visit_count) %>%
  # Remove participants who only have accelerometer AND lab data for a single visit
  filter(num_accel_visits > 1 & final_sppb_score_count > 1) %>%
  
  ### Daily aggregates
  # Calculate total steps for each participant at each of the four visits
  group_by(participant_id, visit) %>%
  # Total steps
  mutate(total_steps_all_days = sum(num_steps),
         .after = num_steps) %>%
  
  # Calculate daily aggregates for each participant
  group_by(participant_id, date) %>%
  # Calculate total daily steps
  mutate(daily_steps = sum(num_steps), 
         .after = total_steps_all_days) %>%
  ungroup() %>%
  
  # For each day and participant, calculate steps in each cadence category
  group_by(participant_id, date) %>%
  mutate(
    daily_stepping_steps = sum(num_steps[cadence < 45]),
    daily_slower_steps = sum(num_steps[cadence >= 45 & cadence <= median(cadence[cadence >= 45])]),
    daily_non_fast_steps = daily_stepping_steps + daily_slower_steps,
    daily_faster_steps = sum(num_steps[cadence >= 45 & cadence > median(cadence[cadence >= 45])]),
    .after = daily_steps) %>%
  ungroup() %>%
  
  # Label each event based on whether it's below 45 steps per minute (stepping, not walking),
  # in the lower half (slower walking), or the upper half (faster walking)
  mutate(event_cadence_band = 
           case_when(cadence < 45 ~ 'Stepping, not walking',
                     cadence < median(cadence[cadence >= 45]) ~ 'Slower walking',
                     cadence > median(cadence[cadence >= 45]) ~ 'Faster walking'),
         .after = cadence) %>%
  # Define factor levels
  mutate(event_cadence_band = factor(event_cadence_band, 
                                     levels = c('Stepping, not walking',
                                                'Slower walking',
                                                'Faster walking')))



#######################################
##### Calculate daily aggregates ######
#######################################

# Create new data frame which only contains daily values
accel_daily <- accel %>%
  # Keep only the daily entries
  distinct(participant_id, date, 
           .keep_all = TRUE) %>%
  # Remove duration, num_steps and cadence because they only make sense on event level
  select(-c(duration_s, num_steps, cadence, event_cadence_band))



############################################
##### Calculate visit-level aggregates #####
############################################

accel_visits <- accel_daily %>%
  # Calculate averages for each visit
  group_by(participant_id, visit) %>%
  mutate(visit_total_steps = sum(daily_steps),
         visit_daily_steps_avg = mean(daily_steps),
         visit_stepping_steps_avg = mean(daily_stepping_steps),
         visit_slower_steps_avg = mean(daily_slower_steps),
         visit_non_fast_steps_avg = mean(daily_non_fast_steps),
         visit_faster_steps_avg = mean(daily_faster_steps),
         .after = daily_faster_steps) %>%
  ungroup() %>%
  # Keep each visit only once
  distinct(participant_id, visit,
           .keep_all = TRUE) %>%
  # Remove variables which are not meaningful at visit-level
  select(-c(daily_steps, daily_stepping_steps, daily_slower_steps, daily_non_fast_steps,
            daily_faster_steps, date, measurement_day, weekday, n_hours, n_valid_hours)) %>%
  # Add scaled stepping variables (per 1,000 steps) for better coefficient interpretability
  mutate(visit_daily_steps_1k = visit_daily_steps_avg / 1000,
         visit_stepping_steps_1k = visit_stepping_steps_avg / 1000,
         visit_slower_steps_1k = visit_slower_steps_avg / 1000,
         visit_non_fast_steps_1k = visit_non_fast_steps_avg / 1000,
         visit_faster_steps_1k = visit_faster_steps_avg / 1000,
         .after = visit_faster_steps_avg)

### Create a copy which doesn't use factor names for modelling in STATA
accel_visits_stata <- accel_visits %>%
  mutate(site = case_when(site == 'Bath' ~ 1,
                          site == 'Birmingham' ~ 2,
                          site == 'Exeter' ~ 3),
         visit = case_when(visit == 'Baseline' ~ 1,
                           visit == '6 Months' ~ 2,
                           visit == '12 Months' ~ 3,
                           visit == '24 Months' ~ 4),
         allocation = case_when(allocation == 'Control' ~ 0,
                                allocation == 'Intervention' ~ 1),
         gender = case_when(gender == 'Female' ~ 0,
                            gender == 'Male' ~ 1),
         imd_quintile = case_when(imd_quintile == 'Most deprived (1/5)' ~ 1,
                                  imd_quintile == 'Quite deprived (2/5)' ~ 2,
                                  imd_quintile == 'Mid deprived (3/5)' ~ 3,
                                  imd_quintile == 'Little deprived (4/5)' ~ 4,
                                  imd_quintile == 'Least deprived (5/5)' ~ 5),
         comorbid = case_when(comorbid == 'None' ~ 0,
                              comorbid == 'One or more' ~ 1),
         highest_education = case_when(highest_education == 'Some/all Secondary' ~ 1,
                                       highest_education == 'Some College' ~ 2,
                                       highest_education == 'All College/Degree' ~ 3),
         sf36gen_hlth = case_when(sf36gen_hlth == 'Very good/Excellent' ~ 1,
                                  sf36gen_hlth == 'Good' ~ 2,
                                  sf36gen_hlth == 'Poor/Fair' ~ 3))

### Save a copy as csv
# This is the input for the Stata modelling
write_csv(accel_visits_stata,
          file = '/Users/alexanderschonfelder/Documents/IDSAI/04_Analysis/accel_visits_stata5.csv')



##############################
##### Summary statistics #####
##############################

### Data availability

# Number of participants with valid data
n_participants <- accel_visits %>%
  distinct(participant_id) %>%
  summarise(n = n()) %>%
  pull(n)

# Average number of days with valid data
accel_visits %>%
  group_by(participant_id, visit) %>%
  summarise(visits = sum(num_accel_visits)) %>%
  pull(visits) %>%
  mean()

# Average number of days per visit with valid data
accel_visits %>%
  pull(days_per_visit) %>%
  mean()

# Percentage of participants with data from 2, 3, or 4 visits (% of ALL recruited participants)
accel_daily %>%
  filter(num_accel_visits == 2) %>%
  distinct(participant_id) %>%
  summarise(n = n() / 777)

accel_daily %>%
  filter(num_accel_visits == 3) %>%
  distinct(participant_id) %>%
  summarise(n = n() / 777)

accel_daily %>%
  filter(num_accel_visits == 4) %>%
  distinct(participant_id) %>%
  summarise(n = n() / 777)

# Descriptive statistics for age and SPPB
accel_visits %>%
  distinct(participant_id, 
           .keep_all = TRUE) %>%
  mutate(final_sppb_score = as.numeric(final_sppb_score)) %>%
  select(age, final_sppb_score) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        na.rm = TRUE,
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')



### Event-level summaries

# Event-level metrics
accel %>%
  select(c(num_steps, duration_s, cadence)) %>%
  rename('Cadence (steps/min)' = cadence,
         'Duration (seconds)' = duration_s,
         'Step count' = num_steps) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

### Event-level metrics by cadence band
accel %>%
  # Select variables to appear in the table
  select(c(num_steps, duration_s, cadence, event_cadence_band)) %>%
  # Give them a name that's nicer to read
  rename('Event step count' = num_steps,
         'Event duration (sec)' = duration_s,
         'Event cadence' = cadence) %>%
  # Group by cadence category
  group_by(event_cadence_band) %>%
  # Select descriptive statistics
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        na.rm = TRUE,
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

# Event-level metrics by visit
accel %>%
  select(c(visit, num_steps, duration_s, cadence)) %>%
  rename('Cadence (steps/min)' = cadence,
         'Duration (seconds)' = duration_s,
         'Step count' = num_steps) %>%
  group_by(visit) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')

# Event-level metrics by sex
accel %>%
  select(c(gender, visit, num_steps, duration_s, cadence)) %>%
  rename('Cadence (steps/min)' = cadence,
         'Duration (seconds)' = duration_s,
         'Step count' = num_steps) %>%
  group_by(gender) %>%
  descr(stats = c('mean', 'sd', 'min', 'q1', 'med', 'q3', 'max'),
        order = 'preserve',
        transpose = TRUE,
        justify = 'r',
        style = 'rmarkdown')



### Table 1
accel_visits %>%
  distinct(participant_id,
           .keep_all = TRUE) %>%
  mutate(final_sppb_score = as.numeric(final_sppb_score)) %>%
  select(age, gender, final_sppb_score, imd_quintile, highest_education, comorbid, 
         sf36gen_hlth, allocation, site) %>%
  tbl_summary(by = gender,
              statistic = list(all_continuous() ~ '{mean} ({sd})'),
              missing_text = 'n/a',
              label = c(age ~ 'Age', 
                        final_sppb_score ~ 'SPPB Score',
                        imd_quintile ~ 'IMD Quintile',
                        highest_education ~ 'Highest Education',
                        comorbid ~ 'Comorbidity',
                        sf36gen_hlth ~ 'SF-36 General Health Score',
                        allocation ~ 'Allocation', 
                        site ~ 'Site')) %>%
  add_overall() %>%
  bold_labels() %>%
  modify_header(label ~ '**Variable**')



### Step counts at each visit

# Visit 1
tbl1 <- accel_visits %>%
  filter(visit == 'Baseline') %>%
  distinct(participant_id, visit,
           .keep_all = TRUE) %>%
  select(visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_non_fast_steps_avg, visit_faster_steps_avg) %>%
  tbl_summary(statistic = list(all_continuous() ~ '{mean} ({sd})'),
              missing_text = 'n/a',
              label = c(visit_daily_steps_avg ~ 'Total steps',
                        visit_stepping_steps_avg ~ 'Non-walking steps',
                        visit_non_fast_steps_avg ~ 'Slower-paced steps',
                        visit_slower_steps_avg ~ 'Slower-paced walking steps',
                        visit_faster_steps_avg ~ 'Faster-paced walking steps')) %>%
  modify_header(label ~ '**Variable**')

# Visit 2
tbl2 <- accel_visits %>%
  filter(visit == '6 Months') %>%
  distinct(participant_id, visit,
           .keep_all = TRUE) %>%
  select(visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_non_fast_steps_avg, visit_faster_steps_avg) %>%
  tbl_summary(statistic = list(all_continuous() ~ '{mean} ({sd})'),
              missing_text = 'n/a',
              label = c(visit_daily_steps_avg ~ 'Total steps',
                        visit_stepping_steps_avg ~ 'Non-walking steps',
                        visit_non_fast_steps_avg ~ 'Slower-paced steps',
                        visit_slower_steps_avg ~ 'Slower-paced walking steps',
                        visit_faster_steps_avg ~ 'Faster-paced walking steps')) %>%
  modify_header(label ~ '**Variable**')

# Visit 3
tbl3 <- accel_visits %>%
  filter(visit == '12 Months') %>%
  distinct(participant_id, visit,
           .keep_all = TRUE) %>%
  select(visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_non_fast_steps_avg, visit_faster_steps_avg) %>%
  tbl_summary(statistic = list(all_continuous() ~ '{mean} ({sd})'),
              missing_text = 'n/a',
              label = c(visit_daily_steps_avg ~ 'Total steps',
                        visit_stepping_steps_avg ~ 'Non-walking steps',
                        visit_non_fast_steps_avg ~ 'Slower-paced steps',
                        visit_slower_steps_avg ~ 'Slower-paced walking steps',
                        visit_faster_steps_avg ~ 'Faster-paced walking steps')) %>%
  modify_header(label ~ '**Variable**')

# Visit 4
tbl4 <- accel_visits %>%
  filter(visit == '24 Months') %>%
  distinct(participant_id, visit,
           .keep_all = TRUE) %>%
  select(visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_non_fast_steps_avg, visit_faster_steps_avg) %>%
  tbl_summary(statistic = list(all_continuous() ~ '{mean} ({sd})'),
              missing_text = 'n/a',
              label = c(visit_daily_steps_avg ~ 'Total steps',
                        visit_stepping_steps_avg ~ 'Non-walking steps',
                        visit_non_fast_steps_avg ~ 'Slower-paced steps',
                        visit_slower_steps_avg ~ 'Slower-paced walking steps',
                        visit_faster_steps_avg ~ 'Faster-paced walking steps')) %>%
  modify_header(label ~ '**Variable**')

# Merge tables for print
tbl_merge(list(tbl1, tbl2, tbl3, tbl4),
          tab_spanner = c('Baseline', '6 Months', '12 Months', '24 Months'))



### SPPB at baseline

# Mean
accel_visits %>%
  select(participant_id, visit, final_sppb_score) %>%
  filter(visit == 'Baseline') %>%
  summarise(mean_sppb = mean(as.numeric(final_sppb_score)))
# SD
accel_visits %>%
  select(participant_id, visit, final_sppb_score) %>%
  filter(visit == 'Baseline') %>%
  summarise(sd_sppb = sd(as.numeric(final_sppb_score)))

# Mean by sex
accel_visits %>%
  select(participant_id, gender, visit, final_sppb_score) %>%
  filter(visit == 'Baseline') %>%
  group_by(gender) %>%
  summarise(mean_sppb = mean(as.numeric(final_sppb_score)))

# SD by sex
accel_visits %>%
  select(participant_id, gender, visit, final_sppb_score) %>%
  filter(visit == 'Baseline') %>%
  group_by(gender) %>%
  summarise(sd_sppb = sd(as.numeric(final_sppb_score)))





##########################
##### Visualisations #####
##########################

### Visualise split into cadence bands

# All stepping
accel %>%
  # Select cadences and event durations
  select(c('num_steps', 'cadence')) %>%
  # Sort in ascending order for event duration
  arrange(cadence) %>%
  # Add column with cumulative
  mutate(cum_steps = cumsum(num_steps)) %>%
  # Add column with relative cumulative steps
  mutate(rel_cum_steps = cum_steps / sum(num_steps)) %>%
  # Plot
  ggplot(data = ., 
         aes(x = cadence,
             y = rel_cum_steps * 100)) +
  # Annotate Stepping, not walking area
  geom_vline(xintercept = 20,
             colour = 'black',
             linetype = 'dotted') +
  annotate('rect',
           xmin = 20, xmax = 45,
           ymin = -Inf, ymax = Inf,
           fill = 'red', 
           alpha = 0.12) +
  annotate(geom = 'text',
           x = 40,
           y = 0,
           label = 'Non-walking (20-44)',
           colour = 'firebrick3',
           hjust = 0,
           angle = 90) +
  # Annotate Slower Walking area
  geom_vline(xintercept = 45,
             colour = 'black',
             linetype = 'dotted') +
  annotate('rect',
           xmin = 45, xmax = median(accel$cadence[accel$cadence >= 45]),
           ymin = -Inf, ymax = Inf,
           fill = 'green', 
           alpha = 0.12) +
  geom_vline(xintercept = median(accel$cadence[accel$cadence >= 45]),
             colour = 'black',
             linetype = 'dashed') +
  annotate(geom = 'text',
           x = 57,
           y = 0,
           label = 'Slower-paced walking (45-62)',
           colour = 'forestgreen',
           hjust = 0,
           angle = 90) +
  # Annotate Faster Walking area
  annotate('rect',
           xmin = median(accel$cadence[accel$cadence >= 45]), xmax = max(accel$cadence),
           ymin = -Inf, ymax = Inf,
           fill = 'blue', 
           alpha = 0.12) +
  geom_vline(xintercept = max(accel$cadence),
             colour = 'black',
             linetype = 'dotted') +
  annotate(geom = 'text',
           x = 168,
           y = 0,
           label = 'Faster-paced walking (63 - 175)',
           colour = 'dodgerblue3',
           hjust = 0,
           angle = 90) +
  # Draw line for cumulative steps
  geom_line(colour = 'black',
            size = 0.5) +
  # Annotate median cadence
  annotate(geom = 'text',
           x = 64,
           y = 72.5,
           hjust = 0,
           label = paste('Median walking cadence:', round(median(accel$cadence[accel$cadence >= 45]))), 2) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10)) +
  scale_y_continuous(labels = scales::comma,
                     breaks = scales::pretty_breaks(n = 10)) +
  coord_cartesian(xlim = c(0, 175)) +
  labs(title = 'Step accumulation by event cadence',
       x = 'Cadence',
       y = '% of cumulative steps') +
  guides(colour = guide_legend(title = '')) +
  theme_bw()



### How did stepping volume change in Control and Intervention?

# Table
accel_visits %>%
  select(visit, allocation, visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_faster_steps_avg) %>%
  rename('Total steps (20-175)' = visit_daily_steps_avg,
         'Slower-paced steps (20-44)' = visit_stepping_steps_avg,
         'Slower-paced walking steps (45-62)' = visit_slower_steps_avg,
         'Faster-paced walking steps (64-175)' = visit_faster_steps_avg) %>%
  tbl_strata(strata = c('visit', 'allocation'),
             .tbl_fun = ~ .x %>% tbl_summary(statistic = list(all_continuous() ~ '{mean} ({sd})')))

# Plot
accel_visits %>%
  select(visit, allocation, visit_daily_steps_avg, visit_stepping_steps_avg,
         visit_slower_steps_avg, visit_faster_steps_avg) %>%
  group_by(visit, allocation) %>%
  mutate(visit_daily_steps_avg = mean(visit_daily_steps_avg),
         visit_stepping_steps_avg = mean(visit_stepping_steps_avg),
         visit_slower_steps_avg = mean(visit_slower_steps_avg),
         visit_faster_steps_avg = mean(visit_faster_steps_avg)) %>%
  rename('Daily steps (20-175)' = visit_daily_steps_avg,
         'Stepping steps (20-44)' = visit_stepping_steps_avg,
         'Slower steps (45-62)' = visit_slower_steps_avg,
         'Faster steps (64-175)' = visit_faster_steps_avg) %>%
  distinct(visit, allocation,
           .keep_all = TRUE) %>%
  pivot_longer(!c('visit', 'allocation'), 
               names_to = 'stepping_var',
               values_to = 'avg_steps') %>%
  ggplot(., aes(x = visit, 
                y = avg_steps,
                colour = allocation,
                group = allocation)) +
  geom_point(alpha = 0.5) +
  geom_line(alpha = 0.5) +
  labs(title = 'Mean daily steps at each visit',
       x = 'Visit',
       y = 'Mean daily steps') +
  facet_wrap(~ stepping_var) +
  theme_bw() +
  theme(legend.title = element_blank())



### Paginated activity profiles for each participant and visit

# Calculate the number of pages required with 5 columns and 7 rows per page
n_pages <- ceiling(nrow(distinct(accel_daily, participant_id)) / (5 * 7))

# Plot activity profiles for each participant
for (i in seq_len(n_pages)) { 
  print(
    accel_daily %>%
      # Keep only the daily entries
      distinct(participant_id, date, 
               .keep_all = TRUE) %>%
      # Plot
      ggplot(., 
             aes(x = measurement_day, 
                 y = daily_steps, 
                 colour = visit)) +
      geom_line(stat = 'identity',
                alpha = 0.6) +
      geom_point(stat = 'identity',
                 size = 0.9,
                 alpha = 0.6) +
      scale_y_continuous(labels = scales::comma, 
                         breaks = scales::pretty_breaks(n = 4)) +
      scale_x_continuous(breaks = scales::pretty_breaks(n = 7)) +
      labs(title = '',
           x = 'Measurement day', 
           y = 'Steps') +
      guides(colour = guide_legend(title = '')) +
      facet_wrap_paginate(~ participant_id,
                          ncol = 5, 
                          nrow = 7,
                          page = i) +
      theme_bw() +
      theme(legend.position = 'top',
            panel.grid.minor.x = element_blank(),
            panel.grid.major.x = element_blank())
  )
}



### Model 3 estimates (estimates separately obtained with STATA)

estimates <- data.frame(
  variable = factor(
    c('Total steps', 'Total steps 6M', 'Total steps 12M', 'Total steps 24M',
      'Faster-paced walking steps', 'Faster-paced walking steps 6M', 
      'Faster-paced walking steps 12M', 'Faster-paced walking steps 24M', 
      'Slower-paced steps', 'Slower-paced steps 6M',
      'Slower-paced steps 12M', 'Slower-paced steps 24M'),
    levels = c('Total steps', 'Total steps 6M', 'Total steps 12M', 
               'Total steps 24M', 'Faster-paced walking steps', 
               'Faster-paced walking steps 6M', 
               'Faster-paced walking steps 12M', 
               'Faster-paced walking steps 24M',
               'Slower-paced steps', 'Slower-paced steps 6M',
               'Slower-paced steps 12M', 'Slower-paced steps 24M')),
  mod1_estimate = c(0.04, 0.07, 0.12, 0.21, NA, NA, NA, NA, NA, NA, NA, NA),
  mod1_lowerci = c(-0.01, 0.02, 0.07, 0.15, NA, NA, NA, NA, NA, NA, NA, NA),
  mod1_upperci = c(0.08, 0.12, 0.17, 0.26, NA, NA, NA, NA, NA, NA, NA, NA),
  mod2_estimate = c(NA, NA, NA, NA, 0.08, 0.27, 0.43, 0.69, NA, NA, NA, NA),
  mod2_lowerci = c(NA, NA, NA, NA, -0.07, 0.10, 0.25, 0.50, NA, NA, NA, NA),
  mod2_upperci = c(NA, NA, NA, NA, 0.23, 0.44, 0.60, 0.88, NA, NA, NA, NA),
  mod3_estimate = c(NA, NA, NA, NA, 0.05, 0.22, 0.35, 0.53, 0.03, 0.04, 0.06, 0.13),
  mod3_lowerci = c(NA, NA, NA, NA, -0.12, 0.03, 0.15, 0.32, -0.03, -0.03, -0.01, 0.06),
  mod3_upperci = c(NA, NA, NA, NA, 0.13, 0.41, 0.55, 0.74, 0.10, 0.10, 0.13, 0.20))

# Reshape data into tidy data frame
estimates <- pivot_longer(estimates, 
                          !variable,
                          names_to = c('model', '.value'),
                          names_pattern = '(.*)_(.*)',
                          values_to = 'value') %>%
  # Drop entries for which there are no coefficients
  drop_na() %>%
  # Rename models
  mutate(model = case_when(model == 'mod1' ~ 'Model 1',
                           model == 'mod2' ~ 'Model 2',
                           model == 'mod3' ~ 'Model 3'))

# Plot
ggplot(estimates, 
       aes(x = estimate, 
           y = variable, 
           colour = model)) + 
  geom_vline(xintercept = 0,
             colour = 'grey60',
             linetype = 'dashed') +
  # Mod1
  geom_point(data = . %>% filter(model == 'Model 1')) +
  geom_errorbar(data = . %>% filter(model == 'Model 1'),
                aes(xmin = lowerci, 
                    xmax = upperci), 
                width = 0.1) +
  # Mod2
  geom_point(data = . %>% filter(model == 'Model 2'),
             position = position_nudge(y = 0.1)) +
  geom_errorbar(data = . %>% filter(model == 'Model 2'),
                aes(xmin = lowerci, 
                    xmax = upperci), 
                width = 0.1,
                position = position_nudge(y = 0.1)) +
  # Mod3
  geom_point(data = . %>% filter(model == 'Model 3'),
             position = position_nudge(y = -0.1)) +
  geom_errorbar(data = . %>% filter(model == 'Model 3'),
                aes(xmin = lowerci, 
                    xmax = upperci), 
                position = position_nudge(y = -0.1),
                width = 0.1) +
  # Overall settings
  scale_y_discrete(limits = rev(c('Total steps', 'Total steps 6M', 'Total steps 12M', 
                                  'Total steps 24M', 'Faster-paced walking steps', 
                                  'Faster-paced walking steps 6M', 
                                  'Faster-paced walking steps 12M', 
                                  'Faster-paced walking steps 24M',
                                  'Slower-paced steps', 'Slower-paced steps 6M',
                                  'Slower-paced steps 12M', 'Slower-paced steps 24M'))) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 5)) +
  labs(x = 'Coefficient estimates and 95% CIs',
       y = '') +
  coord_cartesian(xlim = c(-0.2,1)) +
  theme_bw() +
  theme(legend.title = element_blank(),
        panel.grid.major.y = element_blank(),
        panel.grid.minor.y = element_blank())



### Contour plot of SPPB changes for different cominations of step count changes

# Define input variables
non_faster_steps <- seq(-1060, 1060, 10)
faster_steps <- seq(-1060, 1060, 10)

coef_non_faster_baseline <- 0.03
coef_faster_baseline <- 0.05

coef_non_faster_24m <- 0.13
coef_faster_24m <- 0.53


# Create permutations and calculate SPPB change
grid <- expand.grid(faster_steps, non_faster_steps) %>%
  rename(faster_steps = Var1,
         non_faster_steps = Var2) %>%
  mutate(sppb_change = (faster_steps * coef_faster_24m + non_faster_steps * coef_non_faster_24m) / 1000)


# Plot

# Option 1 (Annotated contour lines)
ggplot(data = grid,
       aes(x = faster_steps,
           y = non_faster_steps)) +
  geom_contour_filled(aes(x = faster_steps,
                          y = non_faster_steps,
                          z = sppb_change),
                      binwidth = 0.1) +
  geom_vline(xintercept = 0, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_vline(xintercept = 500, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_vline(xintercept = -500, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_vline(xintercept = 1000, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_vline(xintercept = -1000, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_hline(yintercept = 0, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_hline(yintercept = 500, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_hline(yintercept = -500, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_hline(yintercept = 1000, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_hline(yintercept = -1000, colour = 'white', linewidth = 0.25, linetype = 'dashed') +
  geom_textcontour(aes(z = sppb_change),
                   binwidth = 0.1,
                   padding = unit(5, 'pt'),
                   colour = 'black',
                   position = 'nudge') +
  scale_fill_discrete_diverging(palette = 'Blue-Red 2',
                                rev = TRUE) +
  scale_x_continuous(labels = scales::comma) +
  scale_y_continuous(labels = scales::comma) +
  labs(title = 'SPPB change at 24 months vs. baseline',
       x = 'Faster-paced walking steps',
       y = 'Slower-paced steps',
       fill = 'SPPB change') +
  coord_equal() +
  theme_bw() +
  theme(legend.position = 'none',
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank())
